package de.intevation.flys.artifacts;

import java.util.List;

import org.w3c.dom.Document;

import org.apache.log4j.Logger;

import com.vividsolutions.jts.geom.Envelope;

import de.intevation.artifacts.ArtifactFactory;
import de.intevation.artifacts.CallMeta;

import de.intevation.artifactdatabase.state.DefaultOutput;
import de.intevation.artifactdatabase.state.Facet;
import de.intevation.artifactdatabase.state.State;

import de.intevation.flys.model.River;
import de.intevation.flys.model.HydrBoundary;

import de.intevation.flys.artifacts.model.FacetTypes;
import de.intevation.flys.artifacts.model.RiverFactory;
import de.intevation.flys.utils.FLYSUtils;
import de.intevation.flys.utils.GeometryUtils;


public class WMSHydrBoundaryArtifact extends WMSDBArtifact {

    public static final String NAME = "hydr_boundary";


    private static final Logger logger =
        Logger.getLogger(WMSHydrBoundaryArtifact.class);


    @Override
    public void setup(
        String          identifier,
        ArtifactFactory factory,
        Object          context,
        CallMeta        callMeta,
        Document        data)
    {
        logger.debug("WMSHydrBoundaryArtifact.setup");

        super.setup(identifier, factory, context, callMeta, data);
    }


    @Override
    public String getName() {
        return NAME;
    }


    @Override
    public State getCurrentState(Object cc) {
        State s = new HydrBoundaryState(this);

        List<Facet> fs = getFacets(getCurrentStateId());

        DefaultOutput o = new DefaultOutput(
            "floodmap",
            "floodmap",
            "image/png",
            fs,
            "map");

        s.getOutputs().add(o);

        return s;
    }


    public static class HydrBoundaryState extends WMSDBState implements FacetTypes
    {
        private static final Logger logger =
            Logger.getLogger(HydrBoundaryState.class);

        protected int riverId;

        public HydrBoundaryState(WMSDBArtifact artifact) {
            super(artifact);
            riverId = 0;
        }

        @Override
        protected String getFacetType() {
            return FLOODMAP_HYDR_BOUNDARY;
        }

        @Override
        protected String getUrl() {
            return FLYSUtils.getUserWMSUrl(artifact.identifier());
        }

        @Override
        protected String getSrid() {
            River river = RiverFactory.getRiver(getRiverId());
            return FLYSUtils.getRiverSrid(river.getName());
        }

        @Override
        protected Envelope getExtent(boolean reproject) {
            List<HydrBoundary> boundaries;
            String kind = getIdPart(2);
            if (kind != null) {
                boundaries = HydrBoundary.getHydrBoundaries(getRiverId(),
                        getName(), Integer.parseInt(kind));
            } else {
                boundaries = HydrBoundary.getHydrBoundaries(getRiverId(), getName());
            }

            Envelope max = null;

            for (HydrBoundary b: boundaries) {
                Envelope env = b.getGeom().getEnvelopeInternal();

                if (max == null) {
                    max = env;
                    continue;
                }

                max.expandToInclude(env);
            }

            return max != null && reproject
                ? GeometryUtils.transform(max, getSrid())
                : max;
        }

        @Override
        protected String getFilter() {
            String kind = getIdPart(2);
            if (kind != null) {
                return "river_id=" + String.valueOf(getRiverId())
                    + " AND name='" + getName() + "'"
                    + " AND kind = " + kind;
            }
            return "river_id=" + String.valueOf(getRiverId())
                + " AND name='" + getName() + "'";
        }

        @Override
        protected String getDataString() {
            String srid = getSrid();

            if (FLYSUtils.isUsingOracle()) {
                return "geom FROM hydr_boundaries USING SRID " + srid;
            }
            else {
                return "geom FROM hydr_boundaries USING UNIQUE id USING SRID " + srid;
            }
        }

        @Override
        protected String getGeometryType() {
            return "LINE";
        }
    } // end of HydrBoundaryState
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
